/** 
 * Tableau de donnes de type float<br>
 * Cette classe est utilise pour representer une grille 2D de donnes 
 * sous forme d'un tableau
 * 
 *
 * @author HEIT Sylvestre & Coustillac Laurent
 *
 *
 * Programme kartoMNT : gnration de MNT  partir d'une carte de lignes de niveaux
 * Copyright (C) 2002  HEIT Sylvestre & Coustillac Laurent
 * 
 * Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon les termes de la Licence
 *Publique Gnrale GNU publie par la Free Software Foundation (version 2).
 *
 *Ce programme est distribu car potentiellement utile, mais SANS AUCUNE GARANTIE, ni explicite ni implicite,
 *y compris les garanties de commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
 *Licence Publique Gnrale GNU pour plus de dtails.
 *
 *Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme temps que ce programme ; si
 *ce n'est pas le cas, crivez  la
 *Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, tats-Unis.
 */
package kartoMNT;


import java.util.*;
import java.io.*;

/**
 * Tableau de donnees de type float<br>
 * Cette classe est utilisee pour representer une grille 2D de donnees 
 * sous forme d'un tableau
 */
public class FloatData
    {
    	/** Le tableau de donnees */
    	protected float[] data;
		
		/** Dimension en x du tableau */
        protected int xmax;
        
        /** Dimension en Y du tableau */
        protected int ymax;
        
        /** Un ensemble de listeners sur ce tableau */
        Vector listeners;

        /**
         * Constructeur
         * @param x premiere dimension du tableau (X)
         * @param x deuxieme dimension du tableau (Y) 
         */
        public FloatData(int x,int y)
        {
            xmax=x;
            ymax=y;
            data=new float[xmax*ymax];
            listeners=new Vector();
        }

		/**
		 * Diminue la taille du tableau
		 * @param sx facteur de reduction en X
		 * @param sy facteur de reduction en Y
		 */
		public void reduire(float sx, float sy){
         	float[] d = getData();
         	int xmaxOld=xmax;
        	int newXmax=(int) (xmax*sx);
        	int newYmax=(int)(ymax*sy);
        	data=new float[newXmax*newYmax]; 
        	xmax=newXmax;
        	ymax=newYmax;
        	for(int x=0; x<newXmax; x++){
        		for(int y =0; y<newYmax; y++){
        			//setValue(x,y,d[(int)((x/sx)+(xmax*y/sy))]);	
        			setValue(x,y,d[(int)(x+ newXmax*y)]);	
        			
        		}
        	}
        		
    	}
		
		
        void addRegionListener(RegionListener l)
        {
            listeners.add(l);
        }

        void removeRegionListener(RegionListener l)
        {
            listeners.remove(l);
        }

        void clearListener(RegionListener l)
        {
            listeners.clear();
        }

		/**
		 * Fixe le tableau de donnees
		 * @param f le tableau de donnees
		 */
        void setFloatData(float[] f)
        {
            data=f;
            notifyRegionChanged(0,0,xmax,ymax);
        }

        public void notifyRegionChanged(int x0,int y0,int x1,int y1)
        {
            ListIterator it=listeners.listIterator();
            while(it.hasNext())
                {
                    ((RegionListener)it.next()).regionChanged(x0,y0,x1,y1);
                }
        }

        public void notifyRegionChanged()
        {
            notifyRegionChanged(0,0,xmax,ymax);
        }

        protected void notifyResized()
        {
            ListIterator it=listeners.listIterator();
            while(it.hasNext())
                {
                    ((RegionListener)it.next()).resized();
                }
        }


      

		/**
		 * Retourne la valeur du tableau en un point donne
		 * @param x la premiere coordonnee du point (X)
		 * @param y la deuxieme coordonnee du point (Y)
		 * @return la valeur du tableau en (x,y)
		 */
        public float getValue(int x, int y)
        {
            return data[x + xmax * y];
        }

		
        /**
		 * Fixe la valeur du tableau en un point donne
		 * @param x la premiere coordonnee du point (X)
		 * @param y la deuxieme coordonnee du point (Y)
		 * @param value la nouvelle valeur du tableau en (x,y)
		 */
        public void setValue(int x, int y, float value)
        {
            data[x + xmax * y] = value;
        }


		/**
		 * Redimensionne le tableau
		 * @param nx la nouvelle dimension en X
		 * @param ny la nouvelle dimension en Y
		 */
        public void resize(int nx,int ny)
        {
            if((nx==xmax)&&(ny==ymax))
                return;
            float[] d=new float[nx*ny];
            int x,y;
            for(x=0;x<Math.min(nx,xmax);x++)
                for(y=0;y<Math.min(ny,ymax);y++)
                    {
                        d[x+y*nx]=data[x+y*xmax];
                    }
            xmax=nx;
            ymax=ny;
            data=d;
            notifyResized();
        }

		/**
		 * Retourne la dimension en X du tableau
		 * @return dimension en X du tableau
		 */
        public int getXMax()
        {
            return xmax;
        }

        /**
		 * Retourne la dimension en Y du tableau
		 * @return dimension en Y du tableau
		 */
        public int getYMax()
        {
            return ymax;
        }
		
		/**
		 * Renvoie le tableau de donnees
		 * @return le tableau contenant toutes les donnees
		 */
        public float[] getData()
        {
            return data;
        }

		/**
		 * Remplit le tableau a partir d'un DataInputStream
		 * @param dis source des donnees
		 */
        public void read (DataInputStream dis) throws IOException
            {
                xmax=dis.readInt();
                ymax=dis.readInt();
                data=new float[xmax*ymax];
                int i, j;
                for (i = 0; i < xmax; i++)
                    for (j = 0; j < ymax; j++)
                        {
                            setValue(i, j, dis.readFloat());
                        }
            }

	/**
	 * Lecture des donnees du tableau avec un BufferedReader
	 * @param fr le BufferedReader utilise
	 */
	public void read (BufferedReader fr) throws IOException{
		String line = fr.readLine();

		//xmax=fr.readInt();
        //ymax=fr.readInt();
		StringTokenizer st=new StringTokenizer(line);
		xmax=Integer.parseInt(st.nextToken());
		ymax=Integer.parseInt(st.nextToken());
		data=new float[xmax*ymax];
               
        for (int j = 0; j < ymax; j++){
		    line = fr.readLine();
		    st=new StringTokenizer(line);
            for (int i = 0; i < xmax; i++){
           		//setValue(i, j, fr.readFloat());
			    setValue(i, j, Float.parseFloat(st.nextToken()));
			}
		}
    }

	
	/**
	 * Ecriture des donnees dans un DataOutputStream
	 * @param dos la cible
	 */
    public void write (DataOutputStream dos) throws IOException
            {
                int i, j;
                dos.writeInt(xmax);
                dos.writeInt(ymax);
                for (i = 0; i < xmax; i++)
                    for (j = 0; j < ymax; j++)
                        {
                            dos.writeFloat(getValue(i, j));
                        }
            }


        float getMin()
        {
            float min=0;
            if(data.length>0)
                {
                    int i;

                    if(data.length>0)
                        {
                            min=data[0];
                        }
                    for(i=0;i<data.length;i++)
                        {
                            min=Math.min(min,data[i]);

                        }

                }
            return min;
        }

        float getMax()
        {
            float max=-1;
            if(data.length>0)
                {
                    int i;
                    if(data.length>0)
                        {
                            max=data[0];
                        }
                    for(i=0;i<data.length;i++)
                        {
                            max=Math.max(max,data[i]);

                        }

                }
            return max;
        }


        

    }
