package kartoMNT;

import java.io.*;

/**
   Format de MNT
*/
public interface MntFormat extends MNTClassInterface{
    /** indicateur de format d'exportion */
    public static final int EXPORT = 1;
    /** indicateur de format d'importation */
    public static final int IMPORT = 2;

    /**
       Retourne l'extension geree par le format
       @return l'extension correspondant au format
    */
    public String getExtension();

    /**
       Retourne une description du format gere
       @return description du format ger
    */
    public String getDescription();
    /**
     * Mode de support du format
     * @return MntFormat.EXPORT si le format ne supporte que l'exportation, 
               MntFormat.IMPORT si le format ne supporte que l'importation, 
	       MntFormat.EXPORT+MntFormat.IMPORT si le format supporte l'importation et l'exportation
    */
    public int support();

    /**
       Fonction d'exportation du format
       (doit tre vide si le format ne supporte pas l'exportation)
       @param MNT le mnt  exporter
       @param filename nom du fichier o sera sauvegarder le MNT
       @throws  IOException en cas de problme d'criture du fichier
    */
    public void exportMNT (MNT mnt, String filename) throws IOException;

    /**
       Fonction d'importation du format       
       (doit tre vide si le format ne supporte pas l'importation)
       @param filename nom du fichier o sera sauvegarder le MNT
       @return le MNT import
       @throws  IOException en cas de problme de lecture du fichier
    */
    public MNT importMNT(String filename) throws IOException;

}
