package kartoMNT;

import java.io.*;
import java.util.*;

/**
   Format de MNT grant les .sur
*/
public class MntFormatSUR implements MntFormat{

    /** Constructeur */
    public MntFormatSUR(){}

    /**
       Retourne l'extension gre par le format
       @return "sur"
    */
    public String getExtension(){
	return "sur";
    }

    /**
       Retourne une description du format gr
       @return description du format gr
    */
    public String getDescription(){
	return "MNT VTopo";
    }

    /**
     * Mode de support du format
     * @return MntFormat.EXPORT+MntFormat.IMPORT (le format gre l'importation et l'exportation)
    */
    public int support(){
	return MntFormat.EXPORT+MntFormat.IMPORT;
    }

    /**
       Fonction d'exportation du format
       @param MNT le mnt  exporter
       @param filename nom du fichier o sera sauvegarder le MNT
       @throws  IOException en cas de problme d'criture du fichier
    */
    public void exportMNT (MNT mnt, String filename) throws IOException{
	int i, j, xmax = mnt.getXMax(), ymax = mnt.getYMax();
	String eol = System.getProperty("line.separator");
	StringBuffer sb = new StringBuffer (4 * xmax * ymax);
	BufferedWriter bfw = new BufferedWriter (new FileWriter (filename));
	
	sb.append("Surface ").append(mnt.getX1()).append(' ');
	sb.append(mnt.getX2()).append(' ').append(mnt.getPasX()).append(' ');
	sb.append(xmax).append(' ');
	sb.append(mnt.getY1()).append(' ').append(mnt.getY2()).append(' ');
	sb.append(mnt.getPasY()).append(' ').append(ymax).append(eol);
	sb.append(eol); //Saut de ligne
	for (i = 0; i < xmax; i++)
	    {
		for (j = 0; j < ymax; j++)
		    {
			sb.append(' ').append(mnt.getValue(i, j));
		    }
		sb.append(eol);
	    }
	bfw.write (sb.toString());
	bfw.close ();
    }

    /**
       Fonction d'importation du format       
       @param filename nom du fichier o sera sauvegarder le MNT
       @return le MNT import
       @throws  IOException en cas de problme de lecture du fichier
    */
    public MNT importMNT(String filename) throws IOException{
	

	FileInputStream fis = new FileInputStream(filename);	
	BufferedReader br = new BufferedReader (new InputStreamReader(fis));

	int i, j;
	String s;
	String l = br.readLine();
	StringTokenizer st = new StringTokenizer(l," ");
                
	s=st.nextToken();
                
	s=st.nextToken();
	float X1=Float.parseFloat(s);
                
	s=st.nextToken();  //X2
	s=st.nextToken();  
	float pasX=Float.parseFloat(s);
                
	s=st.nextToken();  
	int xmax=Integer.parseInt(s);
                
	s=st.nextToken();
	float Y1=Float.parseFloat(s);
                
	s=st.nextToken();  //Y2
	s=st.nextToken();  
	float pasY=Float.parseFloat(s);
                
	s=st.nextToken();  
	int ymax=Integer.parseInt(s);

	MNT mnt = new MNT(xmax, ymax, X1, Y1, pasX, pasY);
                
	//System.out.println("X1: "+X1+" Y1:"+Y1+" pasX:"+pasX+" pasY:"+pasY+" xmax:"+xmax+" ymax:"+ymax+"\n"); 
		
	float[] data=new float[xmax*ymax];
               				
	String tok;
	i = 0; j = 0;
	while( (l = br.readLine()) != null ){
	    st = new StringTokenizer(l," ");
	    while(st.hasMoreTokens()){
		tok = st.nextToken();
		if( j >= ymax ){
		    i++;
		    j = 0;
		}						
		data[i + xmax * j] = Float.parseFloat(tok);
		j++;
	    }
	}

	mnt.setFloatData(data);

	return mnt;
    }

    /**
       Mthode toString
       @return extension gr par le format
    */
    public String toString(){
	return "Format "+getExtension();
    }

}
