package kartoMNT;

import java.io.*;
import javax.swing.*;

/**
   Format de MNT grant le format VRML (.wrl)
*/
public class MntFormatVRML implements MntFormat{
    /** texture */
    String textureName="";

    /** Constructeur */
    public MntFormatVRML(){}

    /**
       Retourne l'extension gre par le format
       @return "wrl"
    */
    public String getExtension(){
	return "wrl";
    }

    /**
       Retourne une description du format gr
       @return description du format gr
    */
   public String getDescription(){
       return "Reprsentation en VRML";
   }

    /**
     * Mode de support du format
     * @return MntFormat.EXPORT (le format ne gre que l'exportation)
    */
    public int support(){
	return MntFormat.EXPORT;
    }

    /**
       Fonction d'exportation du format
       @param MNT le mnt  exporter
       @param filename nom du fichier o sera sauvegarder le MNT
       @throws  IOException en cas de problme d'criture du fichier
    */
    public void exportMNT (MNT mnt, String filename) throws IOException{
	int i, j, xmax = mnt.getXMax(), ymax = mnt.getYMax();
	float max = mnt.getAltitudeMax();
	String eol = System.getProperty("line.separator");
	StringBuffer sb = new StringBuffer (4000);
	BufferedWriter bfw = new BufferedWriter (new FileWriter (filename));
	sb.append("#VRML V2.0 utf8").append(eol);
	sb.append("#created by KartoMNT on ").append(new java.util.Date(System.currentTimeMillis()).toString());
	if (textureName.length() >0)
	    sb.append(" based on a map file named \"").append(textureName).append("\".").append(eol);
	else
	    sb.append('.').append(eol);
	if (xmax <1 || ymax < 1)
	    {
		sb.append("#that map had no dimensions so this file has nothing to show.").append(eol);
		bfw.write(sb.toString());
		bfw.close();
		return;
	    }// dans la suite on peut considrer xmax et ymax comme strictement positifs
	sb.append(eol);// saut de ligne
	sb.append("Viewpoint {").append(eol);
	sb.append("  position	0 0 0").append(eol);
	sb.append("  fieldOfView	0.6").append(eol);
	sb.append("}").append(eol);
	sb.append("Transform {").append(eol);
	sb.append("  translation ").append(((float) xmax * mnt.pasX / -2.0f)).append(' ').append((int)(-max * 1.1f)).append(' ').append((int) (-1.5*ymax * mnt.pasY)).append(eol);
	sb.append("  children [").append(eol);

	sb.append("  NavigationInfo {headlight TRUE}").append(eol);
	sb.append("  PointLight").append(eol);
	sb.append("     {on TRUE").append(eol);
	sb.append("      intensity 1.0").append(eol);
	sb.append("      ambientIntensity 1.0").append(eol);
	sb.append("      color 1.0 1.0 1.0").append(eol);
	sb.append("		 location 0 500 0").append(eol);
	sb.append("      }").append(eol);
	sb.append("    Shape {").append(eol);
	sb.append("      geometry ElevationGrid {").append(eol);
	sb.append("        xDimension ").append(xmax).append(eol);
	//ATTENTION : en VRML z designe la droite, tandis que y dsigne l'altitude
	sb.append("        zDimension ").append(ymax).append(eol);
	//Au lieu d'un un, on pourrait mettre la taille d'un pixel sur la carte d'origine
	sb.append("        xSpacing ").append(mnt.pasX).append(eol);
	sb.append("        zSpacing ").append(mnt.pasY).append(eol);
	//sb.append("        creaseAngle 180").append(eol);
	sb.append("        height [").append(eol);

	for (j = 0; j < ymax; j++)
	    {
		sb.append("          ").append(mnt.getValue(0, j));

		for (i = 1; i < xmax; i++)
		    {
			sb.append(' ').append(mnt.getValue(i, j));
		    }
		sb.append(eol);
	    }
	sb.append("        ]").append(eol);
	sb.append("      }").append(eol);
	sb.append("      appearance Appearance {").append(eol);
	if (textureName.length() >0)
	    sb.append("      texture ImageTexture {url \"").append(textureName).append("\"}").append(eol);
	sb.append("      material Material {").append(eol);
	sb.append("        emissiveColor 0.0 0.0 0.0").append(eol);
	sb.append("        diffuseColor .1 .8 .3").append(eol);
	sb.append("        specularColor 0 0 0").append(eol);
	sb.append("        shininess 1").append(eol);
	sb.append("        transparency 0").append(eol);
	sb.append("      }").append(eol);
	sb.append("    }").append(eol);
	sb.append("  }").append(eol);
	sb.append(" ]").append(eol);
	sb.append("}").append(eol);
	bfw.write(sb.toString());
	bfw.close();
    }

    /**
       Fonction d'importation du format       
       @param filename nom du fichier o sera sauvegarder le MNT
       @return le MNT import
       @throws  IOException en cas de problme de lecture du fichier
    */
    public MNT importMNT(String filename) throws IOException{
	return null;
    }
   
    /**
       Mthode toString
       @return extension gr par le format
    */
    public String toString(){
	return "Format "+getExtension();
    }

}
